default_config = {"username": None,
                  "password": None,
                  "verbose": True,
                  "save_directory": "/tmp",
                  "whitelist": [],
                  "whitelist_ids": [],
                  "multi_blacklist": [],
                  "multi_whitelist": [],
                  "item": "overview",
                  "sort": "new",
                  "whitelist_distinguished": True,
                  "whitelist_gilded": True,
                  "max_score": 100,
                  "hours": 24,
                  "nuke_hours": 4320,
                  "keep_a_copy": False,
                  "save_directory": None,
                  "trial_run": False,
                  "clear_vote": False,
                  "replacement_format": "random",
                  "edit_only": False,
                  "batch_cooldown": 10}
